%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function mDot = psStdRestr(InputArg)

global psSimStruct
K = psSimStruct.psThermProp.K;
 
Aeff      = InputArg(1);
%mDotUp   = InputArg(2);
pUp       = InputArg(3);
TUp       = InputArg(4);
XgcUp      = InputArg(4+(1:K))';
%mDotDown = InputArg(5+K);
pDown     = InputArg(6+K);
TDown     = InputArg(7+K);
XgcDown    = InputArg(7+K+(1:K))';

% If flow downstreams  pUp > pDown 
if pUp > pDown
    flowSign = 1;
    PI = pDown/pUp;
    [dummy cp cv R] = psThermProp(pUp,TUp,XgcUp,[0 1 1 1 0 0 0 0 0]);
    gamma = cp/cv;
    FK = pUp/sqrt(R*TUp);
else
    flowSign = -1;
    PI = pUp/pDown;
    [dummy cp cv R] = psThermProp(pDown,TDown,XgcDown,[0 1 1 1 0 0 0 0 0]);
    gamma = cp/cv;
    FK = pDown/sqrt(R*TDown);
end
PI = max(PI,(2/(gamma+1))^(gamma/(gamma-1)));
if PI < 0.95
    Psi = sqrt(2*gamma/(gamma-1)*(PI^(2/gamma)-PI^((gamma+1)/gamma)));
else
    PI_old = PI;
    PI = 0.95;
    Psi = sqrt(2*gamma/(gamma-1)*(PI^(2/gamma)-PI^((gamma+1)/gamma)));
    Psi = Psi/(0.95-1)*(PI_old-1);
end

mDot = flowSign*FK*Aeff*Psi;
if isnan(mDot)
    disp('mDot NaN in  psStdrestr.m')
    %    mDot = 1e10;
end
