%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function X0 = psInitCombustion(theta)
% Initializes combustion phase. b = burned zone, u = unburned zone

global psSimStruct

if psSimStruct.psSimPar.giveWarnings
  disp('---')
  disp('Initializing combustion phase')
  %disp('-- press enter --')
  %pause
end

%%% Show progressbar
%showProgress();

% -------------------------------
% Get some parameters
% -------------------------------
initBurnedVolFrac = psSimStruct.psSimPar.initBurnedVolFrac;
deltaT_ad = psSimStruct.psSimPar.deltaT_ad;

p      = psSimStruct.psModel.p0;
T      = psSimStruct.psModel.T0;
Xc_u   = psSimStruct.psModel.Xc0;
[h_u ] = psThermProp(p,T,Xc_u,[1 0 0 0 0 0 0 0 0]);
Xc_b   = psCalcReactXgc(Xc_u);

% -------------------------------
% Calculate temperature and volume
% -------------------------------
% This is an approximation of the first little burned fraction
V  = psVolume(theta); % Fix for driftoff from psVolume
Vb = initBurnedVolFrac*V;
Vu = (1-initBurnedVolFrac)*V;
Tu = T;
% An initial guess
Tb = Tu+deltaT_ad; % *(1-xRes_u)*phi_u;
deltaT = Tb;
%disp('In psInitCombustion: WARNING deltaT_ad is a really crude approximation')
lpCount = 0;
while abs(deltaT) > 1e-10*Tb && lpCount < 100
    [h_b cp_b cv_b R_b] = psThermProp(p,Tb,Xc_b,[1 1 1 1 0 0 0 0 0]);
    deltaT  = -(h_b-h_u)/cp_b;
    Tb = deltaT+Tb;
    lpCount = lpCount+1;
end    
if lpCount >= 100
    disp('Warning: Loop Counter in InitCombustion >= 10')
end

X0 = [p; Vb; Tb; Vu; Tu; Xc_b'; Xc_u'];




