function u = psValve(CA)

% [u] = psValve(CA)
% Calculates the valve states. u = [u1 u2]', where u1
% is the intake valve and u2 the outlet valve.
% u1, u2 = 1 means fully open, u1, u2 = 0 means closed.
%
% This function belongs to the psPack - process simulation program package
% and should not be run on its own.
%   
% psDist/Valve/Generic/psValve.m


CA = mod(CA+2*pi, 4*pi)-2*pi;

global psSimStruct


DIV 	= psSimStruct.psValve.DIV;
DEV 	= psSimStruct.psValve.DEV;

IVLift 	= psSimStruct.psValve.IVLift;
EVLift 	= psSimStruct.psValve.EVLift;

IVO 	= psSimStruct.psValve.IVO;
IVC	= psSimStruct.psValve.IVC;
EVO 	= psSimStruct.psValve.EVO;
EVC 	= psSimStruct.psValve.EVC;

if CA<IVO 
u(1) = 0; 
elseif and(CA>=IVO,CA<IVO+DIV) 
u(1) = 1 - cos((CA-IVO)/DIV*(pi/2))^2; 
elseif and(CA>=IVO+DIV,CA<IVC-DIV) 
u(1) = 1; 
elseif and(CA>=IVC-DIV,CA<IVC) 
u(1) = cos((CA-(IVC-DIV))/DIV*(pi/2))^2; 
elseif CA>=IVC 
u(1) = 0; 
end 

if CA<EVO 
u(2) = 0; 
elseif and(CA>=EVO,CA<EVO+DEV) 
u(2) = 1 - cos((CA-EVO)/DEV*(pi/2))^2; 
elseif and(CA>=EVO+DEV,CA<EVC-DEV) 
u(2) = 1; 
elseif and(CA>=EVC-DEV,CA<EVC) 
u(2) = cos((CA-(EVC-DEV))/DEV*(pi/2))^2; 
elseif CA>=EVC 
u(2) = 0; 
end 

u(1)=u(1)*IVLift; 
u(2)=u(2)*EVLift; 
u=u'; 
