function outValue =  psValveInit(method,arg1,arg2,arg3,arg4,arg5,arg6)
        
% Methods:
%   init                - Initializes psInitStruct
%   initSavedSimulation - Do initialization required for useage
%                         with a saved simulation
%   initData            - Initializes psSimStruct with predefined variables
%   createWindow        - Creates the window-needed.
%   updateWindow        - Updates the window
%   generatePlot        - Generates a plot in the plotting-area.
%   setParameters       - Set simulationParameters for submodule to arg1
%   saveDataClbck       - A callback to save simulationdata to file.
%   doneButtonClbck     - A callback to exit psXXXInit.m
%   cancelButtonClbck   - Another callback to exit psXXXInit.m
%   loadDataClbck       - A callback to load simulationdata from file.
%   changeValClbck      - Do this everytime parameters change
%   changeOptClbck      - Do this everytime an optimbox is clicked
%   changeUnitClbck     - Do this every time a unit-popup is used
%   changeVariantClbck  - Performs actions needed when another
%                         variant of initial data is chosen
%   createFcns          - Creted appropriate functions
%   checkType           - Should return: 'localFile','createLocal','generic'
%   getInitStruct       - Returns the initStruct. (For debugging purposes)
  
% ===========================================================================
%  Global initialization
% ===========================================================================
  persistent psInitStruct  
  global psSimStruct  
  outValue = 0;
  
  if nargin <1,
    disp('Youd should not run this command from the command-prompt.')
    method = '';
    outValue = -1;
  end
    
  
  % Global initialization: Change these according to your own specifications
  psGroup       = 'Valve'; % Example "Geometry" (i.e. not "psGeometry")
  psSubGroup    = 'Generic';
  psPackVersion = '1.0';

  % For plotting
  plotDependentPars = {'EVC', 'EVO', 'IVC', 'IVO',...
           'DEV', 'DIV', 'EVLift', 'IVLift'};
  
  % Method type: Options are 
  %   'localFile'
  %   'createLocal'
  %   'generic'
  methodType = 'generic'; 
  createsLocal = 0;
  
  % Window measures: (Measures in pixels) 
  % Read the documentation to see what the figures mean
  %  * Some distances
  framePixels = 4;
  axesPixels = 60;
  dx = 4 ;
  dy1 = 30;
  dy2 = 8;
  dy3 = 30;
  dy4 = 30; % Could be set to zero if no variants are available in "initData"
  dy5 = 10;
  dy6 = 10;
  y1 = 18;
  y2 = 30;
  x3 = 20;
  %  * Some total distances
  framePartSizeX  = 350;
  graphPartSizeX  = 550;
  graphPartMaxSizeY  = 400;
  
  %  * Some direved measures
  %    Pixels left to divide between the three is
  %    framePartSizeX-2*framePixels-5*dx-x3
  x1 = 115;
  x2 = 107;
  x4 = 80;  
  %  x5+x6 < framePartSizeX-2*framePixels-5*dx-x3
  x5 = 150;
  x6 = 150;
% ===========================================================================
% Methods
% ===========================================================================

  % -------------------------------------------------------------------------
  % Method      : init
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- working directory
  %               arg2 <- isVisual
  %               arg3 <- varStruct
  % -------------------------------------------------------------------------
  % Description : Performs necessary initialization.
  % -------------------------------------------------------------------------
  if strcmp(method,'init')
    if nargin < 2
      outValue = -1; 
      disp('You should specify a working Directory.')
      return;
    end

    % Since there are no global variables in this case
    % psSimStruct.psXXX should be cleared.
    psSimStruct.psValve = [];
    psSimStruct.psValve.varInfo = [];
    psSimStruct.psValve.varInfo.initialized = 0;
    psSimStruct.psValve.varInfo.localCreated = 0;
    
    % Clear the variables.

    psInitStruct = [];
    psInitStruct.varStruct = [];
    psInitStruct.uihandleStruct = [];
    psInitStruct.variantHandling = [];
    psInitStruct.variantHandling.selectedVariant = 1;
    psInitStruct.callbackHandling= [];
    psInitStruct.callbackHandling.recentChangedVars = [];
    psInitSturct.initSettings = [];
    psInitStruct.initSettings.originalSimStruct = [];
    psInitStruct.initSettings.originalSimStruct.varInfo = [];
    psInitStruct.initSettings.originalSimStruct.varInfo.initialized = 0;
    
    % Take care of the in-varaibles
    psInitStruct.initSettings.workingDir = arg1;
    % Create window or not
    if nargin > 2
      psInitStruct.initSettings.isVisual = arg2;
    else
      psInitStruct.initSettings.isVisual = 1;
    end        
    
    %   Find the psValveDir in use
    tmpXXXXLoc = which('psValveInit');
    %       Just a precaution, in case there is a lower/upper problem...
    tmpXXXXName = getfield(dir(tmpXXXXLoc),'name');
    tmpIdx = findstr(tmpXXXXName, tmpXXXXLoc);    
    %       ... and finally we have the thermPropDir
    psInitStruct.initSettings.psValveDir = tmpXXXXLoc(1:(tmpIdx-2));

    % Check if varStruct specified
    if nargin > 3
      tmpVarStruct = arg3;
    else
      tmpVarStruct = [];
    end        

    psInitStruct.initSettings.originalSimStruct = tmpVarStruct;
    psValveInit('initData');
    
    if ~isempty(tmpVarStruct) && ...
          isfield(tmpVarStruct,'varInfo') && ...
          isfield(tmpVarStruct.varInfo,'initialized') && ...
          tmpVarStruct.varInfo.initialized && ...
          isfield(tmpVarStruct.varInfo,'origin') && ...
          strcmp(tmpVarStruct.varInfo.origin,psSubGroup) &&...
          isfield(tmpVarStruct.varInfo,'psPackVersion') && ...
          strcmp(tmpVarStruct.varInfo.psPackVersion,psPackVersion)      
      psInitStruct.varStruct = tmpVarStruct;
    end

    % Make sure that a new window is created, (if it should be)
    psInitStruct.uihandleStruct.mainFig=0;
    
    psValveInit('createWindow');
    outValue = psInitStruct.uihandleStruct.mainFig;
  end
  % -------------------------------------------------------------------------
  % End Method: init
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : initSavedSimulation
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- distDir , arg2 <- workingDir , 
  %               arg3 <- psSimStruct.psXXX
  % -------------------------------------------------------------------------
  % Description : Performes initializations necessary when loading
  %               a simulation. If nothing needs to be done, just
  %               return the psXXX structure. For some cases
  %               this is where the
  %               psSimStruct.psXXX.varInfo.specialInfo is used.
  % -------------------------------------------------------------------------
  if strcmp(method,'initSavedSimulation')
    % Check that the calling of the command is correctly performed.
    if nargin < 4
      outValue = -1;
      disp(['You should specify a distribution-directory'...
            ' and a ' psGroup ' structure.'])
      return;
    end

    % Get variables from arguments
    distDir = arg1;
    workingDir = arg2;
    psValvePar = arg3;
    
    % Do any necessary operations here.

    % Finish upp
    outValue = psValvePar;
  end
  % -------------------------------------------------------------------------
  % End Method: initSavedSimulation
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : initData
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Initializes psInitStruct with predefined values.
  % -------------------------------------------------------------------------
  if strcmp(method, 'initData')         
    psInitStruct.variantHandling.numVariants = 2;
    psInitStruct.variantHandling.variantNames = {'Nissan 2 l',...
                        'Mercedes M112 E36'};
    psInitStruct.varStruct = [];
    switch psInitStruct.variantHandling.selectedVariant;
     case 1
      psInitStruct.varStruct.varInfo.varNames=...
          {'EVC', 'EVO', 'IVC', 'IVO',...
           'DEV', 'DIV', 'EVLift', 'IVLift'};  
      psInitStruct.varStruct.varInfo.varTypes=...
          {'value','value','value','value',...
           'value','value','value','value'};        
      psInitStruct.varStruct.varInfo.units=...
          {{'rad','deg'},{'rad','deg'},{'rad','deg'},{'rad','deg'}...
           {'rad','deg'},{'rad','deg'},{'-','%'},{'-','%'}};  
      psInitStruct.varStruct.varInfo.unitConversionFactors=...
          {{1,pi/180},{1,pi/180},{1,pi/180},{1,pi/180},...
           {1,pi/180},{1,pi/180},{1 , 1e-2},{1 , 1e-2}};
      psInitStruct.varStruct.varInfo.displayedUnitIdx=...
          {2,2,2,2,...
           2,2,2,2};  
      psInitStruct.varStruct.varInfo.varDesc = ...
          {'Exhaust valve closure',...
           'Exhsust valve opening',...
           'Intake valve closure',...
           'Intake valve opening',...
           'Duration intake valve',...
           'Duration exhaust valve',...
           'Lift amount exhaust valve',...
           'Lift amount intake valve'};
      psInitStruct.varStruct.varInfo.depVarNames  ={};% Berkningsordning
      psInitStruct.varStruct.varInfo.depVarFuncs  =...
          {};
      psInitStruct.varStruct.varInfo.freeVarNames =...
          {'EVC', 'EVO', 'IVC', 'IVO',...
           'DEV', 'DIV', 'EVLift', 'IVLift'};
      psInitStruct.varStruct.varInfo.optimVarNames =...
          {'EVC', 'EVO', 'IVC', 'IVO',...
           'DEV', 'DIV', 'EVLift', 'IVLift'};
      psInitStruct.varStruct.varInfo.optimCtrl=...
          {'off','off','off','off',...
           'off','off','off','off'};
      
      psInitStruct.varStruct.varInfo.origin = psSubGroup;
      psInitStruct.varStruct.varInfo.psPackVersion = psPackVersion;
      % Set up the free variables initial values.
      initValuesFreeVars = {(3)/180*pi, (-237)/180*pi, ...
                          (180+55)/180*pi, (-13)/180*pi, ...
                          90/180*pi, 90/180*pi, 1, 1};    
     case 2
      psInitStruct.varStruct.varInfo.varNames=...
          {'EVC', 'EVO', 'IVC', 'IVO',...
           'DEV', 'DIV', 'EVLift', 'IVLift'};  
      psInitStruct.varStruct.varInfo.varTypes=...
          {'value','value','value','value',...
           'value','value','value','value'};        
      psInitStruct.varStruct.varInfo.units=...
          {{'rad','deg'},{'rad','deg'},{'rad','deg'},{'rad','deg'}...
           {'rad','deg'},{'rad','deg'},{'-','%'},{'-','%'}};  
      psInitStruct.varStruct.varInfo.unitConversionFactors=...
          {{1,pi/180},{1,pi/180},{1,pi/180},{1,pi/180},...
           {1,pi/180},{1,pi/180},{1 , 1e-2},{1 , 1e-2}};
      psInitStruct.varStruct.varInfo.displayedUnitIdx=...
          {2,2,2,2,...
           2,2,2,2};  
      psInitStruct.varStruct.varInfo.varDesc = ...
          {'Exhaust valve closure',...
           'Exhsust valve opening',...
           'Intake valve closure',...
           'Intake valve opening',...
           'Duration intake valve',...
           'Duration exhaust valve',...
           'Lift amount exhaust valve',...
           'Lift amount intake valve'};
      psInitStruct.varStruct.varInfo.depVarNames  ={};% Berkningsordning
      psInitStruct.varStruct.varInfo.depVarFuncs  =...
          {};
      psInitStruct.varStruct.varInfo.freeVarNames =...
          {'EVC', 'EVO', 'IVC', 'IVO',...
           'DEV', 'DIV', 'EVLift', 'IVLift'};
      psInitStruct.varStruct.varInfo.optimVarNames =...
          {'EVC', 'EVO', 'IVC', 'IVO',...
           'DEV', 'DIV', 'EVLift', 'IVLift'};
      psInitStruct.varStruct.varInfo.optimCtrl=...
          {'off','off','off','off',...
           'off','off','off','off'};
      
      psInitStruct.varStruct.varInfo.origin = psSubGroup;
      psInitStruct.varStruct.varInfo.psPackVersion = psPackVersion;
      % Set up the free variables initial values.
      initValuesFreeVars = {(25)/180*pi, (-95-180)/180*pi,...
                          (180+55)/180*pi, (-13)/180*pi, ...
                          90/180*pi, 90/180*pi, 1, 1};          
     otherwise
      psInitStruct.variantHandling.selectedVariant = 1;
    end
    
    % Set the variables in the variable-struct
    for ii = 1:length(psInitStruct.varStruct.varInfo.freeVarNames)
      varName = psInitStruct.varStruct.varInfo.freeVarNames{ii};
      psInitStruct.varStruct.(varName) = initValuesFreeVars{ii};
    end
    % Calculate and set the non-free-variables.
    psInitStruct.varStruct = psCalcDepVars(psInitStruct.varStruct);    
  end
  % -------------------------------------------------------------------------
  % End Method: initData
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : createWindow
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Creates and draws necessary objects for the window.
  % -------------------------------------------------------------------------
  if strcmp(method,'createWindow')
    if psInitStruct.initSettings.isVisual
      % If needed, create the window.
      if ~psInitStruct.uihandleStruct.mainFig
        mainFig = figure('Visible','off');
        psInitStruct.uihandleStruct.mainFig = mainFig;
      else
        mainFig = psInitStruct.uihandleStruct.mainFig;
      end

      % Clear the uihandlestruct
      psInitStruct.uihandleStruct.variableList = [];

      % Calculate some necessary variables
      windowSizeX = framePartSizeX+graphPartSizeX;  
      nrBoxes =  length(psInitStruct.varStruct.varInfo.varNames)+1;
      windowSizeY = 2*framePixels+dy1+nrBoxes*y1+(nrBoxes-1)*dy2+dy3...
          +y1+dy4+3*y2+2*dy5+dy6;

      % Make the window look as I wish      
      mainFigPos = get(mainFig,'Position');  
      mainFigPos = [mainFigPos(1) ...
                    mainFigPos(2)+mainFigPos(4)-windowSizeY ...
                    windowSizeX windowSizeY ];
      set(mainFig,'MenuBar','none','Resize','off','Name',...
                  'psValve - Parameter Setup','NumberTitle','off', ...
                  'Position',mainFigPos,'SelectionHighlight','off',...
                  'Units','pixels','Visible','on');

      figure(mainFig);
      clf;

      % Draw the big frame
      uicontrol(mainFig,'Style','frame','Units','pixels',...
                'Position',[framePixels...
                            framePixels...
                            framePartSizeX-2*framePixels...
                            windowSizeY-2*framePixels]);  
      % Draw the lines 
      uicontrol(mainFig,'Style','frame','Units','pixels',...
                'Position',...
                [framePixels,...
                 windowSizeY-framePixels-dy1-y1-dy2-1,...
                 framePartSizeX-2*framePixels,...
                 y1+3]);  


      % Draw the title
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',[framePixels+dx...
                            windowSizeY-framePixels-dy1...
                            framePartSizeX-2*framePixels-2*dx...
                            y1],...
                'String',[psGroup  ' - ' psSubGroup], ...
                'SelectionHighlight','off','TooltipString',...
                'Enter geomtry parameters');  

      % Draw the variable-name title-box
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','Center',...
                'Position',...
                [framePixels+dx...
                 windowSizeY-framePixels-dy1-y1-dy2...
                 x1...
                 y1],...
                'String','Variable Name','SelectionHighlight','off', ...
                'TooltipString','The variable Name'); 
      % Draw the value-title-box followed by ...
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+2*dx+x1,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x2,...
                 y1],...
                'String','Value',...
                'SelectionHighlight','off',...
                'TooltipString','The value of the variable');
      % ... the optim-title-box and ...       
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+3*dx+x1+x2,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x3,...
                 y1],...
                'String','O','SelectionHighlight','off', ...
                'TooltipString',...
                ['Check this box if you want to'...
                 ' let the optimizer routine change this variable.']); 
      
      % ... then the unit-title-box
      uicontrol(mainFig,'Style','text',...
                'Units','pixels','FontWeight','bold',...
                'HorizontalAlignment','center',...
                'Position',...
                [framePixels+4*dx+x1+x2+x3,...
                 windowSizeY-framePixels-dy1-y1-dy2,...
                 x4,...
                 y1],...
                'String','Unit','SelectionHighlight','off', ...
                'TooltipString',...
                ['The selected unit. (Only for displaying purposes)']);
      
      for ii = 1:(length(psInitStruct.varStruct.varInfo.varNames))
        % ------------------------------------------
        % Set some control variables
        % ------------------------------------------

        % Set the main tooltip string
        toolTipString = psInitStruct.varStruct.varInfo.varDesc{ii};
        % Get the name of the variable
        varName = psInitStruct.varStruct.varInfo.varNames{ii};
        % Get the variable type
        varType = psInitStruct.varStruct.varInfo.varTypes{ii};
        % Get the value of the variable
        varValue =  psInitStruct.varStruct.(varName);
        % Set the unit-string and the box-type
        tmpUnits = psInitStruct.varStruct.varInfo.units{ii};
        unIdx = psInitStruct.varStruct.varInfo.displayedUnitIdx{ii};
        unitStr = [];
        if length(tmpUnits) > 1
          for tmpIdx = 1:length(tmpUnits)
            unitStr{tmpIdx} = ['[' tmpUnits{tmpIdx} ']'];
          end
          unitBoxType = 'popupmenu';
        else          
          unitStr=['[' tmpUnits{unIdx} ']'];
          unitBoxType = 'text';
        end
        % Manipulate the value to a fitting string
        switch varType
         case 'table'
          if isnumeric(varValue) && varValue == 0
            varStr = [varName ' table NOT loaded...'];            
          else 
            varStr = [varName ' table loaded...'];
          end
         case 'string'
          if ~isstr(varValue) 
            varStr = [varName ' not Found...'];            
          else 
            varStr = varValue;
          end
         case 'value'
          tmpFactor = ...
              psInitStruct.varStruct.varInfo.unitConversionFactors{ii}{unIdx};
          varStr = num2str(varValue/tmpFactor);
        end
        % Set colour of text and style of valuebox. In case one
        % wants to change to a textbox in the future. Also change
        % the tooltip string if the variable is uneditable
        if find(strcmp(psInitStruct.varStruct.varInfo.freeVarNames,varName))
          valueStyleString = 'edit';
          theColour = [0 0 0];
        else
          valueStyleString = 'edit';
          theColour = [1 0 0];
          toolTipString = [toolTipString ' (Derived/Uneditable parameter)'];
        end
        % Get optimCtrlStr
        optimVarIdx = ...
            find(strcmp(psInitStruct.varStruct.varInfo.optimVarNames,varName));
        isOptimVar = ~isempty(optimVarIdx);
        if isOptimVar
          optimCtrlStr = ...
              psInitStruct.varStruct.varInfo.optimCtrl{optimVarIdx};
        else 
          optimCtrlStr = '';
        end

        % ------------------------------------------
        % Start drawing
        % ------------------------------------------
        
        % Draw the variable Name
        uicontrol(mainFig,'Style','text',...
                  'Units','pixels',...
                  'HorizontalAlignment','left',...
                  'Position',...
                  [framePixels+dx...
                   windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2)...
                   x1...
                   y1],...
                  'String',varName,'SelectionHighlight','off', ...
                  'TooltipString',toolTipString); 
        % Draw the value box and the unit OR only the variable box
        % Possible types are 'value','table' and 'string'
        switch varType
         case 'table'
          % Draw the value box only since strings or a tables
          % are not optimizable.
          psInitStruct.uihandleStruct.variableList.(varName).value=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','center',...
                        'ForegroundColor',theColour,'Position',...
                        [framePixels+2*dx+x1...
                       windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2)...
                         x2+x3+x4+2*dx...
                         y1],...
                        'Enable','off',...
                        'String',varStr,...
                        'Callback','psValveInit(''changeValClbck'');',...
                        'SelectionHighlight','off',...
                        'TooltipString','Uneditable table or struct loaded');
         case 'string'
          % Draw the value box only since strings or a tables
          % are not optimizable.
          psInitStruct.uihandleStruct.variableList.(varName).value=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','center',...
                        'ForegroundColor',theColour,'Position',...
                        [framePixels+2*dx+x1...
                       windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2)...
                         x2+x3+x4+2*dx...
                         y1],...
                        'String',varStr,...
                        'Callback','psValveInit(''changeValClbck'');',...
                        'SelectionHighlight','off',...
                        'TooltipString',toolTipString);
         case 'value'
          % Draw the value-box followed by ...
          psInitStruct.uihandleStruct.variableList.(varName).value=...
              uicontrol(mainFig,'Style',valueStyleString,...
                        'Units','pixels',...
                        'HorizontalAlignment','right',...
                        'ForegroundColor',theColour,'Position',...
                        [framePixels+2*dx+x1,...
                      windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x2,...
                         y1],...
                        'String',varStr,...
                        'Callback','psValveInit(''changeValClbck'');',...
                        'SelectionHighlight','off',...
                        'TooltipString',toolTipString);
          % ... the optimbox and ... 
          if isOptimVar                        
          psInitStruct.uihandleStruct.variableList.(varName).opt = ...
              uicontrol(mainFig,'Style','checkbox',...
                        'Units','pixels',...
                        'HorizontalAlignment','left',...
                        'Position',...
                        [framePixels+3*dx+x1+x2,...
                         windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x3,...
                         y1],...
                        'String','','SelectionHighlight','off', ...
                        'Callback','psValveInit(''changeOptClbck'');',...
                        'TooltipString',toolTipString,...
                        'Value',strcmp(optimCtrlStr,'on')); 
          end
          % ... then the unit-box
          psInitStruct.uihandleStruct.variableList.(varName).unit = ...
              uicontrol(mainFig,'Style',unitBoxType,...
                        'Units','pixels','Value',unIdx,...
                        'HorizontalAlignment','center',...
                        'Position',...
                        [framePixels+4*dx+x1+x2+x3,...
                      windowSizeY-framePixels-dy1-y1-dy2-ii*(y1+dy2),...
                         x4,...
                         y1],...
                        'String',unitStr,'SelectionHighlight','off', ...
                        'Callback','psValveInit(''changeUnitClbck'');',...
                        'TooltipString',toolTipString); 
         otherwise 
          h = warndlg(['Not a valid type: ''' varType ''''],'!! Warning !!');
          set(h,'WindowStyle','modal');
          uiwait(h);
        end
      end
      % If there are multiple variants, then draw the variant stuff
      if psInitStruct.variantHandling.numVariants 
        % The Textbox
        toolTipString = 'Select your favourite!';
        uicontrol(mainFig,'Style','text',...
                  'Units','pixels',...
                  'HorizontalAlignment','right',...
                  'Position',...
                  [framePartSizeX-framePixels-2*dx-x6-x5,...
                   framePixels+dy6+3*y2+2*dy5+dy4,...
                   x5,...
                   y1],...
                  'String','Predefined Variants:',...
                  'SelectionHighlight','off','TooltipString',toolTipString);
        % And the popup
        psInitStruct.uihandleStruct.variantSelector=...
            uicontrol(mainFig,'Style','popupmenu',...
                  'Units','pixels',...
                  'HorizontalAlignment','center',...
                  'Position',...
                  [framePartSizeX-framePixels-dx-x6,...
                   framePixels+dy6+3*y2+2*dy5+dy4,...
                   x6,...
                   y1],...
                   'Value',psInitStruct.variantHandling.selectedVariant,...
                   'Callback','psValveInit(''changeVariantClbck'');',...
                  'String',psInitStruct.variantHandling.variantNames,...
                  'SelectionHighlight','off','TooltipString',toolTipString);
      end
      
      % 'Reset' button
      toolTipString = ['Click to reset data for ' psGroup ' (' psSubGroup ')'];
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+dx,...
                 framePixels+dy6+2*y2+2*dy5,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Reset data',...
                'Callback',['psValveInit(''initData'');' ...
                          'psValveInit(''createWindow'');'],...
                'SelectionHighlight','off','TooltipString',toolTipString);

      % 'Load data' button
      toolTipString = 'Click to load parameters from file.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+dx,...
                 framePixels+dy6+y2+dy5,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Load model data',...
                'Callback','psValveInit(''loadDataClbck'');',...
                'SelectionHighlight','off','TooltipString',toolTipString);
      % Save data
      toolTipString = 'Click to save current parameters to file.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+2*dx+(framePartSizeX-2*framePixels-3*dx)/2,...
                 framePixels+dy6+y2+dy5,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Save model data',...
                'Callback','psValveInit(''saveDataClbck'');',...
                'SelectionHighlight','off','TooltipString',toolTipString);

      % Create 'Done' button.
      toolTipString = 'Click to initialize and exit.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+2*dx+(framePartSizeX-2*framePixels-3*dx)/2,...
                 framePixels+dy6,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Done',...
                'Callback','psValveInit(''doneButtonClbck'');',...
                'SelectionHighlight','off','TooltipString',toolTipString);

      % Create 'Cancel' button.
      toolTipString = 'Click to exit without Changing parameters.';
      uicontrol(mainFig,'Style','pushbutton',...
                'Units','pixels',...
                'Position',...
                [framePixels+dx,...
                 framePixels+dy6,...
                 (framePartSizeX-2*framePixels-3*dx)/2,...
                 y2],...
                'String','Cancel',...
                'Callback','psValveInit(''cancelButtonClbck'');',...
                'SelectionHighlight','off','TooltipString',toolTipString);

      if graphPartSizeX > 0
        axesLeft   = (framePartSizeX+axesPixels)/windowSizeX;
        axesWidth  = (graphPartSizeX-2*axesPixels)/windowSizeX;
        if windowSizeY-2*axesPixels < graphPartMaxSizeY 
          tempHeight = windowSizeY-2*axesPixels;
        else
          tempHeight = graphPartMaxSizeY;
        end
        axesHeight = (tempHeight)/windowSizeX;
        axesBottom = (1-axesHeight)/2;
        axesHandle = axes('position',...
                          [axesLeft axesBottom axesWidth axesHeight]);
        psInitStruct.uihandleStruct.plotAxes = axesHandle;
        cla(axesHandle);
        psValveInit('generatePlot',axesHandle);
      end
      
      set(mainFig,'WindowStyle','modal')
    end
  end
  % -------------------------------------------------------------------------
  % End Method: createWindow
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : updateWindow
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Updates the window. For now it only recreates the window.
  % -------------------------------------------------------------------------
  if strcmp(method,'updateWindow') 
    
    % Update main windows value boxes.
    updateList = fieldnames(psInitStruct.uihandleStruct.variableList);
    for kk = 1:length(updateList)
      varName  = updateList{kk};
      varValue = psInitStruct.varStruct.(varName);
      ii = find(strcmp(psInitStruct.varStruct.varInfo.varNames,varName));
      unIdx = psInitStruct.varStruct.varInfo.displayedUnitIdx{ii};
      varType = psInitStruct.varStruct.varInfo.varTypes{ii};
      % Manipulate the value to a fitting string
      switch varType
       case 'table'
        if isnumeric(varValue) && varValue == 0
          varStr = [varName ' table NOT loaded...'];            
        else 
          varStr = [varName ' table loaded...'];
        end
       case 'string'
        if ~isstr(varValue) 
          varStr = [varName ' not Found...'];            
        else 
          varStr = varValue;
        end
       case 'value'
        tmpFactor = ...
            psInitStruct.varStruct.varInfo.unitConversionFactors{ii}{unIdx};
        varStr = num2str(varValue/tmpFactor);
      end
      
      set(psInitStruct.uihandleStruct.variableList.(varName).value,...
          'String',varStr)            
    end

    % Check if something has changed and plot it 
    if isempty(psInitStruct.callbackHandling.recentChangedVars)
      recentChangedVars = [];
    else 
      recentChangedVars = ...
          fieldnames(psInitStruct.callbackHandling.recentChangedVars);
    end    
    if graphPartSizeX > 0 && ...
       ~isempty(recentChangedVars) && ...
       ~isempty(plotDependentPars)
      [A , B] = ...
          meshgrid(1:length(recentChangedVars),...
                   1:length(plotDependentPars));
      idxList1 = reshape(A,1,[]);
      idxList2 = reshape(B,1,[]);
      str1 = {recentChangedVars{idxList1}};
      str2 = {plotDependentPars{idxList2}};
      tmpVl=strcmp(str1,str2);
      if ~isempty(find(tmpVl))
        axesHandle = psInitStruct.uihandleStruct.plotAxes;
        cla(axesHandle)
        psValveInit('generatePlot',axesHandle);      
      end
    end
  end
  % -------------------------------------------------------------------------
  % End Method: updateWindow
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : generatePlot
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- axesHandle
  % -------------------------------------------------------------------------
  % Description : Plots information in the specified axes
  % -------------------------------------------------------------------------
  if strcmp(method,'generatePlot') 
    if nargin < 2
      outValue = -1; 
      disp('You should specify axesHandle to plotfunction.')
      return;
    end
    axesHandle = arg1;    
    axes(axesHandle);    
    
    % Display the current function
    CA = 1:720;
    phi = CA*pi/180;
    for i = 1:length(phi)
      u(i,:) = tmpValveFcn(phi(i),psInitStruct.varStruct)';
    end

    IVTrace = plot(CA, u(:,1), 'b');
    hold on
    EVTrace = plot(CA, u(:,2), 'r');
    axis([CA(1) CA(end) 0 1]);
    set(axesHandle, 'XTick', 180*[0:1:4], 'XTickLabel', {'TC'; 'BC'});
    set(axesHandle, 'YTick',    [0 1],   'YTickLabel', {'closed'; 'open'});
    hold off      
    legend('Intake valve','Exhaust valve')
    title('psValve function (previev)');
    xlabel('Cranc Angle [deg]');
    ylabel('Valve lift [-]');
  end
  % -------------------------------------------------------------------------
  % End Method: generatePlot
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : setParameters
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psSimStruct.psXXX
  % -------------------------------------------------------------------------
  % Description : Set parameters for sub-module to arg1.
  % -------------------------------------------------------------------------
  if strcmp(method, 'setParameters')
    if nargin < 2
      outValue = -1; 
      disp('You should specify new parameters.')
      return;
    end
    newFuncPar = arg1;
    psInitStruct.varStruct = newFuncPar;
    psValveInit('createWindow');
  end  
  % -------------------------------------------------------------------------
  % End Method: setParameters
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : saveDataClbck
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- fileName (optional)
  % -------------------------------------------------------------------------
  % Description : Saves present/current values to file
  % -------------------------------------------------------------------------
  if strcmp(method, 'saveDataClbck') 
    psValveData = psInitStruct.varStruct;

    if nargin < 2      
      [fileName,dirName]=uiputfile('*.mat','Save psValve data to file');
    
    else
      fileName = arg1;
      dirName  = '';
    end
    
    if fileName~=0
      try 
        matlabVer = version;
        if str2num(matlabVer(1)) > 6
          save([dirName fileName], 'psValveData','-V6')
        else  
          save([dirName fileName], 'psValveData')
        end 
      catch
      button = warndlg(['Could not save to file: ' fileName],'Setup psValve');
        uiwait(button);
      end
    else 
      warndlg('Could not save file', 'Setup psValve');
    end      
  end
  % -------------------------------------------------------------------------
  % End Method: saveDataClbck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : doneButtonClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Ends the session by clearing the window, setting
  %               the variables etc.
  % -------------------------------------------------------------------------
  if strcmp(method, 'doneButtonClbck')
    % Clear the psXXX structure.
    psSimStruct.psValve = [];
    
    % Set variables needed for simulation
    switch methodType
     case 'createLocal'
      % In case we are a createLocal function then we should
      % manipulate psFuncPar to a local version. Then we should get
      % all these parameters out here and pass them as arguments, since
      % we want to be able to use the 'createFcns' function without the gui.

      psFuncPar = psInitStruct.varStruct;
      psFuncPar.varInfo.origin = psSubGroup;

      psFuncParLocal = psStripToLocalData(psFuncPar);
      
      workingDir = psInitStruct.initSettings.workingDir;
      % Create the functions
      psValveInit('createFcns',psFuncPar,psFuncParLocal, workingDir);
      
      % Set the signal/message-variable
      psSimStruct.psValve = psFuncParLocal;
      psSimStruct.psValve.varInfo.initialized = 1;
      psSimStruct.psValve.varInfo.localCreated = createsLocal;
     otherwise 
      psSimStruct.psValve = psInitStruct.varStruct;
      psSimStruct.psValve.varInfo.initialized = 1;
      psSimStruct.psValve.varInfo.localCreated = createsLocal;
    end
    
    % Clear out function-stuff.
    mainFig = psInitStruct.uihandleStruct.mainFig;
    if mainFig 
      delete(mainFig);
    end    
    psInitStruct = [];    
  
  end
  % -------------------------------------------------------------------------
  % End Method: doneButtonClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : cancelButtonClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Ends the session by clearing the window, setting
  %               the variables to their original values etc.
  % -------------------------------------------------------------------------
  if strcmp(method, 'cancelButtonClbck')
    
    % Set the psXXX structure to its orignal value
    psSimStruct.psValve = psInitStruct.initSettings.originalSimStruct;

    % Clear out function-stuff.
    mainFig = psInitStruct.uihandleStruct.mainFig;
    if mainFig 
      delete(mainFig);
    end    
    psInitStruct = [];      
  end
  % -------------------------------------------------------------------------
  % End Method: cancelButtonClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : loadDataClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Loads data from file
  % -------------------------------------------------------------------------  
  if strcmp(method, 'loadDataClbck') 
    if nargin < 2      
      [fileName,dirName]=uigetfile('*.mat','Load psValve data from file');
    else 
      fileName = arg1;
      dirName = '';
    end
    
    oldFuncPar = psInitStruct.varStruct;
    if fileName~=0
      try 
        parameters= load([dirName fileName]);
        psInitStruct.varStruct = parameters.psValveData;
        % Just to see that they are all there
        tags = psInitStruct.varStruct.varInfo.varNames;
        for i = 1:length(tags)
          temp = psInitStruct.varStruct.(tags{i});
        end
        isRightOrigin=strcmp(psInitStruct.varStruct.varInfo.origin,...
                               psSubGroup );
        isRightVersion=strcmp(psInitStruct.varStruct.varInfo.psPackVersion,...
                              psPackVersion);
        if ~isRightOrigin || ~isRightVersion
          h = warndlg({['There is a mismatch between origin name ',...
                        'and or version of the creator of the data. ',...
                        'This may, or may not, be a problem. ',...
                        'You should however consider recreating the ',...
                        'data.'],'',...
                       'Another possibility is to manually load the ',...
                       'data file and modify version/origin. (After ',...
                       'checking that your file is correct.)'},...
                      'psPack (Warning)');
        else
          psValveInit('createWindow');
        end
      catch
        button = warndlg(['Could not load file: ' fileName],'Setup psValve');
        psInitStruct.varStruct=oldFuncPar;
        uiwait(button);
      end
    else
      button =warndlg('Could not load file','Setup psValve');
      psInitStruct.varStruct=oldFuncPar;
      uiwait(button);
    end
  end
  % -------------------------------------------------------------------------
  % End Method: loadDataClbck
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : changeValClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Executes operations necessary after a change of a variable
  % -------------------------------------------------------------------------
  if strcmp(method, 'changeValClbck')
    tags = psInitStruct.varStruct.varInfo.varNames;
    for ii = 1:length(tags)
     varTypeIdx=find(strcmp(psInitStruct.varStruct.varInfo.varNames,tags{ii}));
     varType = psInitStruct.varStruct.varInfo.varTypes{varTypeIdx};
     switch varType
       case 'table'
        % Do nothinh
       case 'string'
        oldValue{ii} = psInitStruct.varStruct.(tags{ii});
        psInitStruct.varStruct.(tags{ii})= ...
      get(psInitStruct.uihandleStruct.variableList.(tags{ii}).value, 'String');
       case 'value'
        oldValue{ii} = psInitStruct.varStruct.(tags{ii});
       tmpIdx = find(strcmp(psInitStruct.varStruct.varInfo.varNames,tags{ii}));
        unIdx = psInitStruct.varStruct.varInfo.displayedUnitIdx{tmpIdx};
        tmpFactor = ...
           psInitStruct.varStruct.varInfo.unitConversionFactors{tmpIdx}{unIdx};
        tmpValue = ...
      str2num(get(psInitStruct.uihandleStruct.variableList.(tags{ii}).value,...
                  'String'));
        if isempty(tmpValue)
          tmpValue = oldValue{ii};
        end        
        psInitStruct.varStruct.(tags{ii})= tmpValue*tmpFactor;
      end      
    end

    % Calculate and set the non-free-variables.
    psInitStruct.varStruct = psCalcDepVars(psInitStruct.varStruct);


    % Now check what parameters has really changed and list them
    psInitStruct.callbackHandling.recentChangedVars = [];
    tags = psInitStruct.varStruct.varInfo.varNames;
    for ii = 1:length(tags)
      theVarType = psInitStruct.varStruct.varInfo.varTypes{ii};
      switch theVarType
       case 'value'
        % Since stuff may change due to numerical problems we only
        % consider a value to have changed if a value changes more
        % than 1e-6 relatively. 
	if psInitStruct.varStruct.(tags{ii})
	  if abs(oldValue{ii}/psInitStruct.varStruct.(tags{ii})-1) > 1e-6
	    psInitStruct.callbackHandling.recentChangedVars.(tags{ii}) = 1;
	  end
	else
	  if oldValue{ii} ~= 0
	    psInitStruct.callbackHandling.recentChangedVars.(tags{ii}) = 1;
	  end
	end
       case 'table'
        % Do nothing
       case 'string'
        % Unlikely but possibly do we want to update plot when a
        % string value changes.
        if strcmp(oldValue{ii},psInitStruct.varStruct.(tags{ii}))
          psInitStruct.callbackHandling.recentChangedVars.(tags{ii}) = 1;
        end
       otherwise
        % Should complain...
      end
    end
    psValveInit('updateWindow');
  end
  % -------------------------------------------------------------------------
  % End Method: changeValClbck
  % -------------------------------------------------------------------------
  
  % -------------------------------------------------------------------------
  % Method      : changeOptClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Executes operations needed after a opt-box-click
  % -------------------------------------------------------------------------
  if strcmp(method, 'changeOptClbck')
    tags = psInitStruct.varStruct.varInfo.optimVarNames;
    for ii = 1:length(tags)
      % Now, the only thing that is needed is a translation from
      % value to string
      if get(psInitStruct.uihandleStruct.variableList.(tags{ii}).opt,'Value')
        psInitStruct.varStruct.varInfo.optimCtrl{ii} = 'on';
        %disp('on')
      else
        psInitStruct.varStruct.varInfo.optimCtrl{ii} = 'off';
        %disp('off')
      end
    end    
      % No need to update the window i should think
      %  psValveInit('updateWindow');
  end  
  % -------------------------------------------------------------------------
  % End Method: changeOptClbck
  % -------------------------------------------------------------------------  

  % -------------------------------------------------------------------------
  % Method      : changeUnitClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Executes operations needed after a unitchange
  % -------------------------------------------------------------------------
  if strcmp(method,'changeUnitClbck')
    % Sort out which fieldnames there are in the figpar
    tmpFieldNames = fieldnames(psInitStruct.uihandleStruct.variableList);
    for ii = 1:length(tmpFieldNames)
      varName = tmpFieldNames{ii};
      % Now if it is a unit that is changeable ... 
   if isfield(psInitStruct.uihandleStruct.variableList.(varName),'unit') && ...
strcmp(get(psInitStruct.uihandleStruct.variableList.(varName).unit,'Style'),...
                   'popupmenu')
        % ... we should sort out what idx the variable has and ...
        theIdx = find(strcmp(psInitStruct.varStruct.varInfo.varNames,varName));
        % ... set the variables displayedUnitIdx.
        psInitStruct.varStruct.varInfo.displayedUnitIdx{theIdx} =...
          get(psInitStruct.uihandleStruct.variableList.(varName).unit,'Value');
      end      
    end    
    
    psValveInit('updateWindow');
  end  
  % -------------------------------------------------------------------------
  % End Method: changeUnitClbck
  % -------------------------------------------------------------------------  
  
  % -------------------------------------------------------------------------
  % Method      : changeVariantClbck
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Performs actions needed when another variant of
  %               intial data is selected
  % ------------------------------------------------------------------------- 
  if strcmp(method,'changeVariantClbck')
    oldVariant = psInitStruct.variantHandling.selectedVariant;
    psInitStruct.variantHandling.selectedVariant = ...
        get(psInitStruct.uihandleStruct.variantSelector,'Value');
    if psInitStruct.variantHandling.selectedVariant ~= oldVariant
      psValveInit('initData');
      psValveInit('createWindow'); % Relly needs to re-create
                                       % the window since the layout
                                       % might have changed.
    end
  end
  % -------------------------------------------------------------------------
  % End Method: changeVariantClbck
  % -------------------------------------------------------------------------
    
  % -------------------------------------------------------------------------
  % Method      : createFcns
  % -------------------------------------------------------------------------
  % Arguments   : arg1 <- psSimStruct.psXXX
  %               arg2 <- psVarStructLocal
  %               arg3 <- workingDir 
  % -------------------------------------------------------------------------
  % Description : Creates local functions for sub-module
  % -------------------------------------------------------------------------
  if strcmp(method,'createFcns')  
    disp('Unimplemented')
  end
  % -------------------------------------------------------------------------
  % End Method: createFcns
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : checkType
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Returns the type of the psXXXInit.m
  % -------------------------------------------------------------------------
  if strcmp(method,'checkType')
    outValue = methodType;
  end  
  % -------------------------------------------------------------------------
  % End Method: checkType
  % -------------------------------------------------------------------------

  % -------------------------------------------------------------------------
  % Method      : getInitStruct
  % -------------------------------------------------------------------------
  % Arguments   : 
  % -------------------------------------------------------------------------
  % Description : Return the initStruct. (For debugging purposes.)
  % -------------------------------------------------------------------------
  if strcmp(method,'getInitStruct')
    outValue = psInitStruct;
  end  
  % -------------------------------------------------------------------------
  % End Method: getInitStruct
  % -------------------------------------------------------------------------

% ===========================================================================  
% Functions
% ===========================================================================

% -----------------------------------------
% Function: Function to generate currentValveFucntion
% -----------------------------------------
function u=tmpValveFcn(CA,funcPar)  
% [u] = tmpValveFcn(CA)  
% Calculates the valve states. u = [u1 u2]', where u1
% is the intake valve and u2 the outlet valve.
% u1, u2 = 1 means fully open, u1, u2 = 0 means closed.

CA = mod(CA, 4*pi)-2*pi;

tags    = {'DEV','DIV','EVLift','IVLift','EVC','EVO','IVC','IVO'};
for i = 1:length(tags)
  eval([tags{i} '= funcPar.(tags{i});']);
end

if CA<IVO 
u(1) = 0; 
elseif and(CA>=IVO,CA<IVO+DIV) 
u(1) = 1 - cos((CA-IVO)/DIV*(pi/2))^2; 
elseif and(CA>=IVO+DIV,CA<IVC-DIV) 
u(1) = 1; 
elseif and(CA>=IVC-DIV,CA<IVC) 
u(1) = cos((CA-(IVC-DIV))/DIV*(pi/2))^2; 
elseif CA>=IVC 
u(1) = 0; 
end 

if CA<EVO 
u(2) = 0; 
elseif and(CA>=EVO,CA<EVO+DEV) 
u(2) = 1 - cos((CA-EVO)/DEV*(pi/2))^2; 
elseif and(CA>=EVO+DEV,CA<EVC-DEV) 
u(2) = 1; 
elseif and(CA>=EVC-DEV,CA<EVC) 
u(2) = cos((CA-(EVC-DEV))/DEV*(pi/2))^2; 
elseif CA>=EVC 
u(2) = 0; 
end 

u(1)=u(1)*IVLift; 
u(2)=u(2)*EVLift; 
u=u'; 
